@@ -201,7 +201,7 @@ class dlgAbsolute(QtWidgets.QDialog):#,dlgAbsoluteIui.Ui_dlgSAXSAbsolute):
             #print self.params
             #self.parentwindow.data_dict[self.datasetname].parameters=self.params
             self.close()
-        elif name=="Apply":
+        elif "pply" in name : #name=="Apply":
             self.onParamChanged()
             #apply
             #-- on wich data set ?
