@@ -18,7 +18,7 @@ from matplotlib.backends.backend_qt5agg import FigureCanvasQTAgg as FigureCanvas
 import matplotlib.patches as patches
 
 from pySAXS.models import SpherePolyWBC
-from conda.common._logic import FALSE
+#from conda.common._logic import FALSE
 MODEL=SpherePolyWBC.SphereOTBackgdBC()
 MODEL_PARAM=[0,1,2]
 ICON_PATH=pySAXS.__path__[0]+os.sep+'guisaxs'+os.sep+'images'+os.sep 
