@@ -67,7 +67,7 @@ FROM_EDF=['Comment','x','z','count_time','pilroi0','pilai1']
 FROM_RPT=['filename','samplex','samplez',"exposure","transmitted flux","incident flux"]
 
 #IMAGE_PARAMS={"edf":FROM_EDF,"tiff":FROM_RPT}
-IMAGE_TYPE=['*.edf*','*.TIFF*','*.tif','*.tiff','*.mar2300']
+IMAGE_TYPE=['*.edf*','*.TIFF*','*.tif','*.tiff','*.mar2300','*.h5']
 
 
 
