@@ -1,6 +1,7 @@
 from PyQt5 import QtCore, QtGui, QtWidgets, uic
 from PyQt5.QtCore import QAbstractTableModel,Qt
 
+
 #from PyQt4.Qt import QString
 from fileinput import filename
 from pyFAI import azimuthalIntegrator
@@ -13,6 +14,7 @@ import matplotlib.colors as colors
 from pySAXS.tools import FAIsaxs
 from pySAXS.tools import filetools
 import os
+import shutil
 import sys
 from scipy import ndimage
 if sys.version_info.major>=3:
@@ -26,6 +28,7 @@ from matplotlib.patches import Circle
 from PyQt5 import QtTest
 from pySAXS.guisaxs import pySaxsColors
 import pandas as pd
+import pandas
 from scipy import interpolate
 
 
@@ -140,6 +143,7 @@ class XeussSurveyorDialog(QtWidgets.QDialog):
         #self.ui.setupUi(self)
         #self.ui.paramFileButton.clicked.connect(self.OnClickparamFileButton)
         self.ui.changeDirButton.clicked.connect(self.OnClickchangeDirButton)
+        self.ui.btnCopyFiles.clicked.connect(self.OnClickCopyFiles)
         #QtCore.QObject.connect(self.ui.STARTButton, QtCore.SIGNAL("clicked()"), self.OnClickSTARTButton)
         #QtCore.QObject.connect(self.ui.STOPButton, QtCore.SIGNAL("clicked()"), self.OnClickSTOPButton)
         #self.ui.plotChkBox.clicked.connect(self.OnClickPlotCheckBox)
@@ -367,7 +371,8 @@ class XeussSurveyorDialog(QtWidgets.QDialog):
         #print(self.df_xeussConfig)
         #print(header)
         try:
-            source=float(header["Wavelength"]  )          #S2 is on s2hl + s2hr
+            #source=float(header["Wavelength"]  )          #S2 is on s2hl + s2hr
+            source = round(float(header['Wavelength']), 15)  # round with 4 digits
         except:
             #print('error Source type not found')
             return None, None
@@ -429,6 +434,43 @@ class XeussSurveyorDialog(QtWidgets.QDialog):
         self.OnClickButtonTransferParams()
         self.radialPrepare()
 
+    def OnClickCopyFiles(self):
+        '''
+        User clicked to copy files on another directory
+        '''
+        #1- ask for directory
+        fd = QtWidgets.QFileDialog(self)
+        destinationDir = fd.getExistingDirectory(self, "Open Directory", \
+                                      directory=self.workingdirectory)#,options=QFileDialog.ShowDirsOnly| QFileDialog.DontResolveSymlinks)
+        print(destinationDir)
+        #2- get the selected files
+        #3- copy the files
+        for item in self.ui.tableWidget.selectedIndexes():
+
+            row=item.row()
+            name=str(self.ui.tableWidget.item(row,0).text())
+            #self.SelectedFile=name
+            #processName=""
+            #print(name)
+            #fullname=self.workingdirectory+os.sep+name
+            #print(name)
+            # try to copy
+            src_path=self.workingdirectory+os.sep+name
+            dst_path=destinationDir+os.sep+name
+            try:
+                shutil.copy(src_path, dst_path)
+                print(src_path + " ->" + dst_path+" succeeded !")
+                src_path_edf=src_path[:-3]+"edf"
+                dst_path_edf=dst_path[:-3]+"edf"
+                if os.path.exists(src_path_edf):
+                    shutil.copy(src_path_edf, dst_path_edf)
+                    print(src_path_edf + " ->" + dst_path_edf + " succeeded !")
+
+            except :
+                print('failed to copy the files')
+                print(src_path+" ->"+dst_path)
+
+
     def OnClickChangeXeussConfigFile(self):
         '''
         Allow to select a parameter file
@@ -541,13 +583,16 @@ class XeussSurveyorDialog(QtWidgets.QDialog):
         display the image
         '''
         ind=self.ui.tabWidget.currentIndex()
-        if ind==0:
-            self.displayPLot()
-        elif ind==1:
-            self.displayImage()
-        else:
-            self.displayPLot()
-            self.displayImage()
+        try:
+            if ind==0:
+                self.displayPLot()
+            elif ind==1:
+                self.displayImage()
+            else:
+                self.displayPLot()
+                self.displayImage()
+        except:
+            pass
 
 
     def getQIDatas(self,fi):
@@ -564,9 +609,20 @@ class XeussSurveyorDialog(QtWidgets.QDialog):
         #print("header size %i" %n )
         dat = numpy.loadtxt(fi, comments='#', skiprows=n+1, encoding='utf8')
         dat = numpy.transpose(numpy.array(dat))
+
         q = dat[0]
         i = dat[1]
         s = dat[2]
+        #removing nan values
+        isnotNan = numpy.where(~numpy.isnan(i))
+        q = q[isnotNan]
+        i = i[isnotNan]
+        s = s[isnotNan]
+        #nan_mask = numpy.isnan(i)
+        #nan_indices = numpy.where(nan_mask)
+        #print(nan_indices)
+        #if len(nan_indices)>0:
+        #    print("number of nan %i"%len(nan_indices))
         return q,i,s
 
     def displayPLot(self):
@@ -994,7 +1050,7 @@ class XeussSurveyorDialog(QtWidgets.QDialog):
             print('error')
             return None, None, None
 
-        self.fai = FAIsaxs.FAIsaxs()
+
         head = self.img.header
         dd = float(head['SampleDistance']) * 10  # m->cm
         centerx = float(head['Center_1'])
@@ -1008,21 +1064,24 @@ class XeussSurveyorDialog(QtWidgets.QDialog):
         self.faiNewConfig=[dd,centerx,centery,tilt,tiltPlanRotation,pixelsize]
         if self.faiConfig!=self.faiNewConfig:
                 #recalculate fai geometry
+                self.fai = FAIsaxs.FAIsaxs()
                 self.fai.set_wavelength(wavelength)
                 self.fai.setFit2D(dd, centerX=centerx, centerY=centery, tilt=tilt, \
                                   tiltPlanRotation=tiltPlanRotation, \
                                   pixelX=pixelsize, pixelY=pixelsize)
 
     def radialPrepareAndAverage(self):
+        t0 = time()
         self.radialPrepare()
 
         ## get the mask
         if self.mad is None:
             self.readMaskEDF()
 
-        t0 = time()
+
         #    print "Error plot"
-        qdiv = 1000  # float(self.ui.edit_qdiv.text())
+        qdiv = 1000
+        qdiv= int(self.ui.edtPyFAINbOfPoints.text())
         qdiv = int(qdiv)
         qtemp, itemp, stemp = self.fai.integrate1d(self.img.data, qdiv, mask=self.mad.data, error_model="poisson",
                                                unit="q_A^-1", )
@@ -1032,6 +1091,8 @@ class XeussSurveyorDialog(QtWidgets.QDialog):
         q = qtemp[numpy.nonzero(itemp)]
         i = itemp[numpy.nonzero(itemp)]
         s = stemp[numpy.nonzero(itemp)]
+        t1=time()
+        #print('radial averaged in %6.4f s'%(t1-t0))
         return q,i,s
 
     def radialAverage(self, imageFilename,plotRefresh=True):
@@ -1075,6 +1136,13 @@ class XeussSurveyorDialog(QtWidgets.QDialog):
         param=None
         #get the list
         #ll=[]
+
+        #self.ui.chkCutQRange.isChecked()
+        if self.ui.chkMerging.isChecked():
+            #try different go to merce process
+            self.tryMerge()
+
+            return
         n=self.ui.tableWidget.rowCount()
         self.ui.progressBar.setMaximum(n)
 
@@ -1091,6 +1159,8 @@ class XeussSurveyorDialog(QtWidgets.QDialog):
             i_work = self.DATAS[name]['i']
             s_work = self.DATAS[name]['s']
             self.DATAS[name].update({'q_work': q_work, 'i_work': i_work, 's_work': s_work})
+
+            # from EDF
             if self.ui.radioButton_EDF.isChecked():
                 imagefile = name[:-3] + "edf"
 
@@ -1113,14 +1183,23 @@ class XeussSurveyorDialog(QtWidgets.QDialog):
                         #print(param.parameters)
                         #print(param.get('D'))
                         head = self.img.header
-                        dd = float(head['SampleDistance']) * 100  # m->cm
-                        param.set('D',dd)
+                        #print(head)
+                        try:
+                            dd = float(head['SampleDistance']) * 100  # m->cm
+                            param.set('D',dd)
+                        except:
+                            print("cannot get the detector distance from edf header")
                         #FOR_ABSOLUTE = ['D', 'TransmittedFlux', 'thickness', 'time', 'wavelength', 'pixel_size']
                         #EDFHEADER_TO_ABSOLUTE = ['SampleDistance', 'TransmittedFlux', 'Thickness', 'ExposureTime','Wavelength', 'PSize_1']
                         #param.calculate_All()
-                        param.set('TransmittedFlux', float(head['TransmittedFlux']))
-                        if float(head['Thickness'])!=0:
-                            param.set('thickness', float(head['Thickness']))
+                        #print(head)
+                        try:
+                            param.set('TransmittedFlux', float(head['TransmittedFlux']))
+                        except:
+                            print("cannot get the Transmitted Flux from edf header")
+                            param.set('TransmittedFlux',1.0)
+                        #if float(head['Thickness'])!=0:
+                        #    param.set('thickness', float(head['Thickness']))
                         param.set('time', float(head['ExposureTime']))
                         param.set('wavelength', float(head['Wavelength']))
                         param.set('pixel_size', float(head['PSize_1'])*100) #m ->cm
@@ -1188,6 +1267,272 @@ class XeussSurveyorDialog(QtWidgets.QDialog):
         self.ui.progressBar.setValue(0)
         #self.ui.chkAutomaticAV.setChecked(st)
 
+    def tryMerge(self):
+
+        n = self.ui.tableWidget.rowCount()
+        self.ui.progressBar.setMaximum(n)
+        param=None
+        colorMerged=None
+        #getting a dataframe with all the files & headers
+        fichiers_edf=[]
+        for item in self.ui.tableWidget.selectedIndexes():
+            row = item.row()
+            name = str(self.ui.tableWidget.item(row, 0).text())
+            fichiers_edf.append(name)
+            #print("will merge %s"%name)
+        #print('will process : ')
+        #print(fichiers_edf)
+
+        first = True
+        for fname in fichiers_edf:
+
+            im = fabio.open(self.workingdirectory + os.sep + fname[:-3]+"edf")
+            head = im.header
+            new_row = pandas.DataFrame([head])
+            new_row['file_name'] = fname
+            new_row['DAT_file_name'] = fname[:-3]
+            if first:
+                df_edf = new_row
+                first = False
+            else:
+                df_edf = pandas.concat([df_edf, new_row], ignore_index=True)
+        # df_edf : dataframe with all header
+        #self.df_xeussConfig the config
+        numeric_columns = ['s2hl', 's2hr', 'SampleDistance', 'Wavelength']
+        df_edf[numeric_columns] = df_edf[numeric_columns].apply(pandas.to_numeric, errors='coerce')
+        df_edf['S2'] = df_edf['s2hl'] + df_edf['s2hr']
+        df_edf['SampleDistance'] = df_edf['SampleDistance'].round(4)
+        df_edf['Wavelength'] = df_edf['Wavelength'].round(15)
+        # try to join the tables
+        merged_df = pandas.merge(df_edf, self.df_xeussConfig, on=['S2', 'SampleDistance', 'Wavelength'], how='left')
+        #sort the samples
+        samples = merged_df['Comment'].unique()
+        print("samples to merge :")
+        print(samples)
+        #Data_SAMPLE = merged_df[merged_df['Comment'].isin(samples)]
+        #Data_SAMPLE_sorted = Data_SAMPLE.sort_values(by='qmin_precise')
+        pbValue=0
+        self.ui.progressBar.setValue(pbValue)
+        for sample_name in samples:
+            pbValue +=1
+            self.ui.progressBar.setValue(pbValue)
+            Data_SAMPLE = merged_df[merged_df['Comment'] == sample_name]
+            Data_SAMPLE_sorted = Data_SAMPLE.sort_values(by='qmin_precise')
+            #print(Data_SAMPLE_sorted['qmin_precise'])
+            colorMerged = pySaxsColors.pySaxsColors().getColor(len(self.parent.data_dict))  # get a new color
+            for i in range(len(Data_SAMPLE_sorted)):
+                #try if conf is found
+
+                row = Data_SAMPLE_sorted.iloc[i]
+                name = row['file_name']
+                print('processing  '+str(name))
+                print(row['configName'])
+                try:
+                    test=row['configName']+'-'+name
+                except:
+                    #no config found
+                    print('no config found: failed')
+                    break
+
+
+                #get iq data
+
+                q_work = self.DATAS[name]['q']
+                i_work = self.DATAS[name]['i']
+                s_work = self.DATAS[name]['s']
+                self.DATAS[name].update({'q_work': q_work, 'i_work': i_work, 's_work': s_work})
+                datfilename = row['DAT_file_name']
+                #q0, i0, e0 = getQIDatas(datfilename)
+                t0 = time()
+                if self.ui.radioButton_EDF.isChecked():
+
+                    imagefile = name[:-3] + "edf"
+
+                    if not os.path.exists(self.workingdirectory + os.sep + imagefile):
+                        print('edf file not exist in the directory : ')
+                    else:
+                        try:
+                            fi = self.workingdirectory + os.sep + imagefile
+                            self.img = fabio.open(fi)  # Open image file
+                            # -------- pyFAI
+                            print('radial averaging :' + fi)
+                            q_work, i_work, s_work = self.radialPrepareAndAverage()
+                            self.DATAS[name].update({'q_work': q_work, 'i_work': i_work, 's_work': s_work})
+                            self.DATAS[name].update({'q_edf': q_work, 'i_edf': i_work, 's_edf': s_work})
+                        except:
+                            print('error when processing file ' + fi)
+
+                    #
+                    param = SAXSparameters.SAXSparameters()
+                    if param is not None:
+                        # param.
+                        # print(param.parameters)
+                        # print(param.get('D'))
+                        head = self.img.header
+                        # print(head)
+                        try:
+                            dd = float(head['SampleDistance']) * 100  # m->cm
+                            param.set('D', dd)
+                        except:
+                            print("cannot get the detector distance from edf header")
+                        # FOR_ABSOLUTE = ['D', 'TransmittedFlux', 'thickness', 'time', 'wavelength', 'pixel_size']
+                        # EDFHEADER_TO_ABSOLUTE = ['SampleDistance', 'TransmittedFlux', 'Thickness', 'ExposureTime','Wavelength', 'PSize_1']
+                        # param.calculate_All()
+                        # print(head)
+                        try:
+                            param.set('TransmittedFlux', float(head['TransmittedFlux']))
+                        except:
+                            print("cannot get the Transmitted Flux from edf header")
+                            param.set('TransmittedFlux', 1.0)
+                        # if float(head['Thickness'])!=0:
+                        #    param.set('thickness', float(head['Thickness']))
+                        param.set('time', float(head['ExposureTime']))
+                        param.set('wavelength', float(head['Wavelength']))
+                        param.set('pixel_size', float(head['PSize_1']) * 100)  # m ->cm
+                    t1=time()
+                    print("edf opened in %6.4fs"%(t1-t0))
+
+                conf = row['configName']
+                qmin = float(row['qmin_large'])
+                qmax = float(row['qmax_merging'])
+                # print('%s : qmin= %s qmax= %s'%(conf,row['qmin_large'],row['qmax_merging']))
+                q_work, i_work, s_work = self.clipQmin(q_work, i_work, s_work, qmin)
+                '''
+                if i < (len(Data_SAMPLE_sorted) - 1):
+                    q_work, i_work, s_work = self.clipQmax(q_work, i_work, s_work, qmax)
+                self.DATAS[name].update({'q_work': q_work, 'i_work': i_work, 's_work': s_work})
+                '''
+                # create new datas in pySAXS
+                myname = row['Comment']
+
+                if myname in self.parent.data_dict:
+                    # keep the color
+                    color = self.parent.data_dict[myname].color
+                else:
+                    if colorMerged is not None:
+                        color=colorMerged
+                    else:
+                        color = pySaxsColors.pySaxsColors().getColor(len(self.parent.data_dict))  # get a new color
+
+                #self.parent.data_dict[myname] = dataset.dataset(myname, q_work, i_work, \
+                #                                                error=s_work, type='saxs',
+                #                                                image="Image", color=color)
+
+                self.parent.data_dict[myname] = dataset.dataset(myname, q_work, i_work, \
+                                                                error=s_work, type='saxs',
+                                                                color=color)
+                if param is not None:
+                    newname_scaled = dlgAbsoluteI.OnScalingSAXSApply(self.parent, dataname=myname,
+                                                                     parameters=param.parameters)
+                    self.parent.data_dict[myname].parameters = param
+                    self.parent.data_dict[myname].checked = False
+                    t1 = time()
+                    print("scaling in %6.4fs" % (t1 - t0))
+                self.PROCESSED[myname] = name  # keep on memory the processed data
+
+                #concatenate
+                if param is not None:
+                    q1 = self.parent.data_dict[newname_scaled].q
+                    i1 = self.parent.data_dict[newname_scaled].i
+                    e1 = self.parent.data_dict[newname_scaled].error
+                else:
+                    q1 = self.parent.data_dict[myname].q
+                    i1 = self.parent.data_dict[myname].i
+                    e1 = self.parent.data_dict[myname].error
+                if i == 0:
+                    #firsts datas
+                    q_final = q1
+                    i_final = i1
+                    e_final = e1
+                    cut2=qmax
+                else:
+                    # merging
+                    '''
+                    q_final = numpy.concatenate((q_final, q1))
+                    i_final = numpy.concatenate((i_final, i1))
+                    e_final = numpy.concatenate((e_final, e1))
+                    '''
+                    cut1=qmin
+                    #print('cut1 = %6.2f cut2 = %6.2f'%(cut1, cut2))
+                    q_final, i_final, e_final = self.merging(q_final, i_final, e_final, q1, i1, e1, cut1, cut2)
+                    cut2=qmax
+                    #nan_mask = numpy.isnan(i_final)
+                    # Print the boolean array
+                    #print("Boolean array indicating NaN values:")
+                    #print(nan_mask)
+                    # Print the indices of NaN values
+                    #nan_indices = numpy.where(nan_mask)
+                    #print("\nIndices of NaN values:")
+                    #print(nan_indices)
+            #after all the data from the same sample
+            #sort the q
+            '''sortedIndexes = numpy.argsort(q_final)
+            q_sorted = q_final[sortedIndexes]
+            i_sorted = i_final[sortedIndexes]
+            e_sorted = e_final[sortedIndexes]
+            '''
+            t1 = time()
+            print("merging in %6.4fs" % (t1 - t0))
+            merged_name=self.DATAS[name]['Comment']+"_merged"
+            self.parent.data_dict[merged_name] = dataset.dataset(merged_name, q_final, i_final, \
+                                                            error=e_final)
+            self.parent.data_dict.pop(myname)
+            if param is not None:
+                if newname_scaled in self.parent.data_dict:
+                    self.parent.data_dict.pop(newname_scaled)
+
+            # -------- SUBTRACT SOLVENT
+            if self.ui.chkSubSolvent.isChecked():
+                solventName = str(self.ui.solventEdit.text())
+                self.subtractSolventRaw(merged_name, solventName)
+
+        self.parent.redrawTheList()
+        self.parent.Replot()
+        # process last for plot refreshing
+
+        self.ui.progressBar.setValue(0)
+
+    def merging(self,q0, i0, e0, q1, i1, e1, cut1, cut2):
+        '''
+        merging two iq curve
+        mean between cut1 & cut2
+        '''
+        # print("start of merge : from %6.4f"%(q0[0]))
+        # cut1=dataArray[j+1][5]
+        # print("cut1 %6.4f"%cut1)
+        # cut2=dataArray[j][6]
+        # print("cut2 %6.4f"%cut2)
+        # first datas
+        # print("first data")
+        # print("end of merge : from %6.4f"%(dataArray[j][1][-1]))
+        # 1 part from actual qmin to cut1
+        q_final, i_final, e_final = self.clipQmax(q0, i0, e0, cut1)
+
+        # 2 part from next qmin large to actual qmax merging
+        # to be merge with next datas
+        q2, i2, e2 = self.clipQmin(q0, i0, e0, cut1)
+        q2, i2, e2 = self.clipQmax(q2, i2, e2, cut2)
+
+        # 3 part from second qmin large to
+        q3, i3, e3 = self.clipQmin(q1, i1, e1, cut1)
+        q3, i3, e3 = self.clipQmax(q3, i3, e3, cut2)
+        # MEAN
+        newf = interpolate.interp1d(q2, i2, kind='linear', bounds_error=False,fill_value='extrapolate')
+        newfe = interpolate.interp1d(q2, e2, kind='linear', bounds_error=False,fill_value='extrapolate')
+        newi = newf(q3)
+        newe = newfe(q3)
+
+        q_final = numpy.concatenate((q_final, q3))
+        i_final = numpy.concatenate((i_final, (newi + i3) / 2))
+        e_final = numpy.concatenate((e_final, (newe + e3)))
+
+        # 4 part
+        q4, i4, e4 = self.clipQmin(q1, i1, e1, cut2)
+        q_final = numpy.concatenate((q_final, q4))
+        i_final = numpy.concatenate((i_final, i4))
+        e_final = numpy.concatenate((e_final, e4))
+        return q_final, i_final, e_final
+
     def clipQmin(self,q,i,error,qmin):
         # clip q min
         if error is not None:
@@ -1196,6 +1541,80 @@ class XeussSurveyorDialog(QtWidgets.QDialog):
         q = numpy.repeat(q, q >= qmin)
         return q,i,error
 
+    def clipQmax(self, q, i, error, qmax):
+        # clip q max
+        if error is not None:
+            error = numpy.repeat(error, q <= qmax)
+        i = numpy.repeat(i, q <= qmax)
+        q = numpy.repeat(q, q <= qmax)
+        return q, i, error
+
+    def subtractSolventRaw(self,dataName,solventName):
+
+        '''
+        '''
+        if not(dataName in self.parent.data_dict):
+            print('no data %s'%dataName)
+            return
+        qSample = self.parent.data_dict[dataName].q
+        iSample=self.parent.data_dict[dataName].i
+        sSample=self.parent.data_dict[dataName].i
+
+        #verify the solvent
+        solventExist=False
+        if solventName in self.parent.data_dict:
+            qSolvent = self.parent.data_dict[solventName].q
+            iSolvent = self.parent.data_dict[solventName].i
+            sSolvent = self.parent.data_dict[solventName].error
+            solventExist = True
+        else:
+            print("no solvent in datas")
+            return
+
+        if solventName==dataName:
+            print('solvent = data')
+            return
+
+        if len(iSample) != len(iSolvent):
+            # trying interpolation
+            print("subtraction with interpolation")
+            newSolvent = interpolate.interp1d(qSolvent, iSolvent, kind='linear',
+                                              bounds_error=0)  # interpolation for i
+            iSolvent = newSolvent(qSample)
+
+            if sSolvent is not None:
+                newErr = interpolate.interp1d(qSolvent, sSolvent, kind='linear',
+                                              bounds_error=0)  # interpolation for i
+                sSolvent = newErr(qSample)
+            qSolvent = qSample
+        '''
+        if  'Thickness' in self.DATAS[dataName]:
+            thickness=float(self.DATAS[dataName]['Thickness'])
+            if thickness==0.0:
+                thickness=1.0
+        else:
+            thickness=1.0
+        '''
+        thickness=1.0
+        iFinal = (iSample - iSolvent) / thickness
+        #print("thickness= %6.2f"%thickness)
+        #print("-------------------------------------            calculate uncertainties")
+        sFinal = (sSample + sSolvent) / thickness
+        '''
+        self.DATAS[dataName].update({'q_work': qSample, 'i_work': iFinal, 's_work': sFinal})
+        self.DATAS[dataName].update({'q_sub': qSample, 'i_sub': iFinal, 's_sub': sFinal})
+        self.DATAS[dataName]['solvent_data']=solventName
+        '''
+        sub_name = dataName + "_subtracted"
+        self.parent.data_dict[sub_name] = dataset.dataset(sub_name, qSolvent, iFinal, \
+                                                             error=sFinal)
+        #self.parent.data_dict.pop(myname)
+
+        self.parent.redrawTheList()
+        self.parent.Replot()
+
+        return
+
     def subtractSolvent(self,dataName,solventName):
 
         qSample = self.DATAS[dataName]['q_work']
