@@ -261,51 +261,52 @@ class mainGuisaxs(QtWidgets.QMainWindow):
         submenuDict={}
         toolbarListPluginsActions=[]
         for name in pl:
-            try:
+                    #try:
+                    #print(pl)
                     m=self.my_import(base+name) #import plugins
                     cl= m.classlist             #the the class list from the plugin
-                    #print cl
+                    #print( cl)
                     for c in cl:
-                        try:
-                            o=getattr(m,c)          #create an object 
-                            #objlist.append(o)
-                            #print o.menu,o.subMenu,o.subMenuText #get the menus
-                            sub=o.subMenu
-                            item=o
-                            '''if o.subMenuText=='':
-                                #no submenu, just an action
-                                action = self.ui.menuData_Treatment.addAction(o.subMenu) #add in the menu
-                                self.connect(action,QtCore.SIGNAL('triggered()'), lambda item=item: self.callPlugin(item))
-                            else:'''
-                            if sub not in submenuDict:
-                                    #submenu doesn t exist
-                                    itemSub = QtWidgets.QMenu(self.ui.menuData_Treatment)
-                                    itemSub.setObjectName(sub)
-                                    itemSub.setTitle(sub)
-                                    if o.icon is not None:
-                                        icon1 = QtGui.QIcon()
-                                        icon1.addPixmap(QtGui.QPixmap(pySAXS.ICON_PATH+o.icon), QtGui.QIcon.Normal, QtGui.QIcon.On)
-                                        itemSub.setIcon(icon1)
-                                    submenuDict[sub]=itemSub
-                            #submenu exist
-                            itemSub=submenuDict[sub]
-                            action = itemSub.addAction(o.subMenuText) #add in the menu
-                            if o.icon is not None:
-                                #print "add icon ",o.icon, pySAXS.ICON_PATH+o.icon
-                                icon1 = QtGui.QIcon()
-                                icon1.addPixmap(QtGui.QPixmap(pySAXS.ICON_PATH+o.icon), QtGui.QIcon.Normal, QtGui.QIcon.On)
-                                action.setIcon(icon1)
-                            self.ui.menuData_Treatment.addAction(itemSub.menuAction())
-                            action.triggered.connect(partial(self.callPlugin,o))
-                            if o.toolbar:
-                                #print "add to toolbar"
-                                toolbarListPluginsActions.append(action)
-                        except:# AttributeError :
-                            print("Unexpected error :", sys.exc_info()[0])
-                            print("module : ",c ," module will not be available")
-            except:# AttributeError:
-                        print("Unexpected error:", sys.exc_info()[0])
-                        print("module : ",name ," module will not be available")
+                        #try:
+                        o=getattr(m,c)          #create an object
+                        #objlist.append(o)
+                        #print o.menu,o.subMenu,o.subMenuText #get the menus
+                        sub=o.subMenu
+                        item=o
+                        '''if o.subMenuText=='':
+                            #no submenu, just an action
+                            action = self.ui.menuData_Treatment.addAction(o.subMenu) #add in the menu
+                            self.connect(action,QtCore.SIGNAL('triggered()'), lambda item=item: self.callPlugin(item))
+                        else:'''
+                        if sub not in submenuDict:
+                                #submenu doesn t exist
+                                itemSub = QtWidgets.QMenu(self.ui.menuData_Treatment)
+                                itemSub.setObjectName(sub)
+                                itemSub.setTitle(sub)
+                                if o.icon is not None:
+                                    icon1 = QtGui.QIcon()
+                                    icon1.addPixmap(QtGui.QPixmap(pySAXS.ICON_PATH+o.icon), QtGui.QIcon.Normal, QtGui.QIcon.On)
+                                    itemSub.setIcon(icon1)
+                                submenuDict[sub]=itemSub
+                        #submenu exist
+                        itemSub=submenuDict[sub]
+                        action = itemSub.addAction(o.subMenuText) #add in the menu
+                        if o.icon is not None:
+                            #print "add icon ",o.icon, pySAXS.ICON_PATH+o.icon
+                            icon1 = QtGui.QIcon()
+                            icon1.addPixmap(QtGui.QPixmap(pySAXS.ICON_PATH+o.icon), QtGui.QIcon.Normal, QtGui.QIcon.On)
+                            action.setIcon(icon1)
+                        self.ui.menuData_Treatment.addAction(itemSub.menuAction())
+                        action.triggered.connect(partial(self.callPlugin,o))
+                        if o.toolbar:
+                            #print "add to toolbar"
+                            toolbarListPluginsActions.append(action)
+                        #except:# AttributeError :
+                        #    print("Unexpected error :", sys.exc_info()[0])
+                        #    print("module : ",c ," module will not be available")
+            #except:# AttributeError:
+            #            print("Unexpected error:", sys.exc_info()[0])
+            #            print("module : ",name ," module will not be available")
         
         #---- Models menu
         if splashScreen is not None:
@@ -2111,8 +2112,8 @@ class mainGuisaxs(QtWidgets.QMainWindow):
     
     def noDataErrorMessage(self):
         QtWidgets.QMessageBox.information(self,"pySAXS", "No datas are selected", buttons=QtWidgets.QMessageBox.Ok, defaultButton=QtWidgets.QMessageBox.NoButton)
-        
-    
+
+
     def OnEditFindPeaks(self):
         '''
         find peaks
@@ -2154,7 +2155,7 @@ class mainGuisaxs(QtWidgets.QMainWindow):
         percent=dlg.percent/100.0
         #end of dialog box    
         '''
-        print(self.plotframe.plotexp)
+        #print(self.plotframe.plotexp)
         #call find peaks
         founds, newq,newi=DetectPeaks.findPeaks(q,i*q**self.plotframe.plotexp,pp,percent,self,method="")
         n=len(founds)
